@@ -1,11 +1,19 @@
 from abc import ABC, abstractmethod
-from collections.abc import Iterable, Iterator
+from collections.abc import Iterator, Sequence
 from dataclasses import dataclass
 from functools import cached_property
 from pathlib import Path
-from typing import Generic
+from typing import Generic, Literal
 
-from cognite.client.data_classes import Asset, Event, FileMetadata, TimeSeries
+from cognite.client.data_classes import (
+    Asset,
+    Event,
+    FileMetadata,
+    FileMetadataList,
+    FileMetadataWrite,
+    FileMetadataWriteList,
+    TimeSeries,
+)
 from cognite.client.data_classes._base import (
     T_CogniteResourceList,
     T_WritableCogniteResource,
@@ -15,41 +23,48 @@ from cognite.client.data_classes._base import (
 )
 from cognite.client.data_classes.data_modeling import EdgeApply, EdgeId, InstanceApply, NodeApply, NodeId
 from cognite.client.utils._identifier import InstanceId
-from rich.console import Console
 
 from cognite_toolkit._cdf_tk.client import ToolkitClient
 from cognite_toolkit._cdf_tk.client.data_classes.instances import InstanceApplyList
+from cognite_toolkit._cdf_tk.client.data_classes.migration import AssetCentricId
 from cognite_toolkit._cdf_tk.client.data_classes.pending_instances_ids import PendingInstanceId
 from cognite_toolkit._cdf_tk.cruds._base_cruds import T_ID
 from cognite_toolkit._cdf_tk.exceptions import ToolkitNotImplementedError
 from cognite_toolkit._cdf_tk.storageio import (
-    AssetCentricSelector,
     BaseAssetCentricIO,
+    FileMetadataIO,
     InstanceIO,
-    InstanceSelector,
-    TableStorageIO,
+    StorageIO,
 )
-from cognite_toolkit._cdf_tk.storageio._base import StorageIOConfig, T_WritableCogniteResourceList
+from cognite_toolkit._cdf_tk.storageio._base import T_WritableCogniteResourceList
+from cognite_toolkit._cdf_tk.storageio.selectors import AssetCentricSelector, InstanceSelector
 from cognite_toolkit._cdf_tk.utils.collection import chunker_sequence
-from cognite_toolkit._cdf_tk.utils.fileio import SchemaColumn
+from cognite_toolkit._cdf_tk.utils.http_client import HTTPClient, HTTPMessage, ItemsRequest, SuccessItem
 from cognite_toolkit._cdf_tk.utils.thread_safe_dict import ThreadSafeDict
 from cognite_toolkit._cdf_tk.utils.useful_types import JsonVal
 
 from .data_classes import MigrationMapping, MigrationMappingList
+from .data_model import INSTANCE_SOURCE_VIEW_ID
 
 
-@dataclass(frozen=True)
 class MigrationSelector(AssetCentricSelector, InstanceSelector, ABC):
     @abstractmethod
     def get_ingestion_views(self) -> list[str]:
         raise NotImplementedError()
 
 
-@dataclass(frozen=True)
 class MigrationCSVFileSelector(MigrationSelector):
+    type: Literal["migrationCSVFile"] = "migrationCSVFile"
     datafile: Path
     resource_type: str
 
+    @property
+    def group(self) -> str:
+        return f"Migration_{self.resource_type}"
+
+    def __str__(self) -> str:
+        return f"file_{self.datafile.name}"
+
     def get_schema_spaces(self) -> list[str] | None:
         return None
 
@@ -62,7 +77,7 @@ class MigrationCSVFileSelector(MigrationSelector):
 
     @cached_property
     def items(self) -> MigrationMappingList:
-        return MigrationMappingList.read_mapping_file(self.datafile, resource_type=self.resource_type)
+        return MigrationMappingList.read_csv_file(self.datafile, resource_type=self.resource_type)
 
 
 @dataclass
@@ -74,8 +89,11 @@ class AssetCentricMapping(Generic[T_WritableCogniteResource], WriteableCogniteRe
         raise NotImplementedError()
 
     def dump(self, camel_case: bool = True) -> dict[str, JsonVal]:
+        mapping = self.mapping.model_dump(exclude_unset=True, by_alias=camel_case)
+        # Ensure that resource type is always included, even if unset.
+        mapping["resourceType" if camel_case else "resource_type"] = self.mapping.resource_type
         return {
-            "mapping": self.mapping.model_dump(exclude_unset=True, by_alias=camel_case),
+            "mapping": mapping,
             "resource": self.resource.dump(camel_case=camel_case),
         }
 
@@ -91,7 +109,7 @@ class AssetCentricMappingList(
 
 class AssetCentricMigrationIOAdapter(
     Generic[T_ID, T_WriteClass, T_WritableCogniteResource, T_CogniteResourceList, T_WritableCogniteResourceList],
-    TableStorageIO[int, MigrationSelector, InstanceApplyList, AssetCentricMappingList],
+    StorageIO[AssetCentricId, MigrationSelector, InstanceApplyList, AssetCentricMappingList],
 ):
     FOLDER_NAME = "migration"
     KIND = "AssetCentricMigration"
@@ -108,22 +126,16 @@ class AssetCentricMigrationIOAdapter(
         base: BaseAssetCentricIO[
             T_ID, T_WriteClass, T_WritableCogniteResource, T_CogniteResourceList, T_WritableCogniteResourceList
         ],
-        instance: InstanceIO,
     ) -> None:
         super().__init__(client)
         self.base = base
-        self.instance = instance
         # Used to cache the mapping between instance IDs and asset-centric IDs.
         # This is used in the as_id method to track the same object as it is converted to an instance.
         self._id_by_instance_id = ThreadSafeDict[InstanceId, int]()
 
-    def get_schema(self, selector: MigrationSelector) -> list[SchemaColumn]:
-        raise ToolkitNotImplementedError("get_schema is not implemented for AssetCentricMigrationIOAdapter")
-
-    @staticmethod
-    def _get_id(item: dict[str, JsonVal]) -> int | None:
+    def _get_id(self, item: dict[str, JsonVal]) -> AssetCentricId | None:
         if "id" in item and isinstance(item["id"], int):
-            return item["id"]
+            return AssetCentricId(self.base.RESOURCE_TYPE, item["id"])
         return None
 
     @staticmethod
@@ -140,27 +152,27 @@ class AssetCentricMigrationIOAdapter(
                 return EdgeId(space=space, external_id=external_id)
         return None
 
-    def as_id(self, item: dict[str, JsonVal] | object) -> int:
+    def as_id(self, item: dict[str, JsonVal] | object) -> AssetCentricId:
         # When multiple threads are accessing this class, they will always operate on different ids
         if isinstance(item, AssetCentricMapping):
             instance_id = item.mapping.instance_id
             self._id_by_instance_id.setdefault(instance_id, item.mapping.id)
-            return self._id_by_instance_id[instance_id]
+            return AssetCentricId(self.base.RESOURCE_TYPE, self._id_by_instance_id[instance_id])
         elif isinstance(item, Event | Asset | TimeSeries | FileMetadata | PendingInstanceId):
             if item.id is None:
                 raise TypeError(f"Resource of type {type(item).__name__!r} is missing an 'id'.")
-            return item.id
+            return AssetCentricId(self.base.RESOURCE_TYPE, item.id)
         elif isinstance(item, NodeApply | EdgeApply):
             instance_id_ = item.as_id()
             if instance_id_ not in self._id_by_instance_id:
                 raise ValueError(f"Missing mapping for instance {instance_id_!r}")
-            return self._id_by_instance_id[instance_id_]
+            return AssetCentricId(self.base.RESOURCE_TYPE, self._id_by_instance_id[instance_id_])
         elif isinstance(item, dict) and (id_int := self._get_id(item)):
             return id_int
         elif isinstance(item, dict) and (parsed_instance_id := self._get_instance_id(item)):
             if parsed_instance_id not in self._id_by_instance_id:
                 raise ValueError(f"Missing mapping for instance {parsed_instance_id!r}")
-            return self._id_by_instance_id[parsed_instance_id]
+            return AssetCentricId(self.base.RESOURCE_TYPE, self._id_by_instance_id[parsed_instance_id])
         raise TypeError(f"Cannot extract ID from item of type {type(item).__name__!r}")
 
     def stream_data(self, selector: MigrationSelector, limit: int | None = None) -> Iterator[AssetCentricMappingList]:
@@ -181,20 +193,67 @@ class AssetCentricMigrationIOAdapter(
     def count(self, selector: AssetCentricSelector) -> int | None:
         return self.base.count(selector)
 
-    def upload_items(self, data_chunk: InstanceApplyList, selector: MigrationSelector) -> None:
-        self.instance.upload_items(data_chunk, selector)
-
     def data_to_json_chunk(self, data_chunk: AssetCentricMappingList) -> list[dict[str, JsonVal]]:
         return data_chunk.dump()
 
     def json_chunk_to_data(self, data_chunk: list[dict[str, JsonVal]]) -> InstanceApplyList:
-        return self.instance.json_chunk_to_data(data_chunk)
+        raise NotImplementedError()
 
-    def load_selector(self, datafile: Path) -> MigrationSelector:
-        raise ToolkitNotImplementedError("load_selector is not implemented for AssetCentricMigrationIOAdapter")
 
-    def configurations(self, selector: MigrationSelector) -> Iterable[StorageIOConfig]:
-        raise ToolkitNotImplementedError("configurations is not implemented for AssetCentricMigrationIOAdapter")
+class FileMetaAdapter(
+    AssetCentricMigrationIOAdapter[
+        str,
+        FileMetadataWrite,
+        FileMetadata,
+        FileMetadataWriteList,
+        FileMetadataList,
+    ]
+):
+    """Adapter for migrating file metadata to data model instances.
 
-    def ensure_configurations(self, selector: MigrationSelector, console: Console | None = None) -> None:
-        raise ToolkitNotImplementedError("ensure_configurations is not implemented for AssetCentricMappingList")
+    This is necessary to link asset-centric FileMetadata to their new CogniteFile instances using the
+    files/set-pending-instance-ids.
+    """
+
+    def __init__(self, client: ToolkitClient) -> None:
+        super().__init__(client, FileMetadataIO(client))
+
+    @staticmethod
+    def as_pending_instance_id(item: InstanceApply) -> PendingInstanceId:
+        """Convert an InstanceApply to a PendingInstanceId for linking."""
+        source = next((source for source in item.sources if source.source == INSTANCE_SOURCE_VIEW_ID), None)
+        if source is None:
+            raise ValueError(f"Cannot extract ID from item of type {type(item).__name__!r}")
+        if not isinstance(source.properties["id"], int):
+            raise ValueError(f"Unexpected ID type: {type(source.properties['id']).__name__!r}")
+        id_ = source.properties["id"]
+        return PendingInstanceId(
+            pending_instance_id=NodeId(item.space, item.external_id),
+            id=id_,
+        )
+
+    def upload_items(
+        self, data_chunk: InstanceApplyList, http_client: HTTPClient, selector: MigrationSelector | None = None
+    ) -> Sequence[HTTPMessage]:
+        """Upload items by first linking them using files/set-pending-instance-ids and then uploading the instances."""
+        config = http_client.config
+        results: list[HTTPMessage] = []
+        successful_linked: set[int] = set()
+        for batch in chunker_sequence(data_chunk, self.CHUNK_SIZE):
+            batch_results = http_client.request_with_retries(
+                message=ItemsRequest(
+                    endpoint_url=config.create_api_url("files/set-pending-instance-ids"),
+                    method="POST",
+                    api_version="alpha",
+                    items=[self.as_pending_instance_id(item).dump() for item in batch],
+                    as_id=self.as_id,
+                )
+            )
+            for res in batch_results:
+                if isinstance(res, SuccessItem):
+                    successful_linked.add(res.id)
+            results.extend(batch_results)
+        to_upload = [item for item in data_chunk if self.as_id(item) in successful_linked]
+        if to_upload:
+            results.extend(list(super().upload_items(InstanceApplyList(to_upload), http_client, selector)))
+        return results
