@@ -81,7 +81,7 @@ EnvType: TypeAlias = Literal["dev", "test", "staging", "qa", "prod"]
 USE_SENTRY = "pytest" not in sys.modules and os.environ.get("SENTRY_ENABLED", "true").lower() == "true"
 SPACE_FORMAT_PATTERN = r"^[a-zA-Z][a-zA-Z0-9_-]{0,41}[a-zA-Z0-9]?$"
 DM_EXTERNAL_ID_PATTERN = r"^[a-zA-Z]([a-zA-Z0-9_]{0,253}[a-zA-Z0-9])?$"
-INSTANCE_EXTERNAL_ID_PATTERN = r"^[^\\x00]{1,256}$"
+INSTANCE_EXTERNAL_ID_PATTERN = r"^[^\x00]{1,256}$"
 FORBIDDEN_SPACES = frozenset(["space", "cdf", "dms", "pg3", "shared", "system", "node", "edge"])
 FORBIDDEN_CONTAINER_AND_VIEW_EXTERNAL_IDS = frozenset(
     [
@@ -160,3 +160,8 @@ COGNITE_MIGRATION_SPACE = "cognite_migration"
 COGNITE_TIME_SERIES_CONTAINER = ContainerId("cdf_cdm", "CogniteTimeSeries")
 COGNITE_FILE_CONTAINER = ContainerId("cdf_cdm", "CogniteFile")
 CDF_UNIT_SPACE = "cdf_cdm_units"
+
+# Data Plugin Constants
+DATA_DEFAULT_DIR = "data"
+DATA_RESOURCE_DIR = "resources"
+DATA_MANIFEST_STEM = "Manifest"
