@@ -68,7 +68,6 @@ from cognite.client.data_classes.data_modeling.ids import (
     NodeId,
     ViewId,
 )
-from cognite.client.data_classes.data_modeling.views import ReverseDirectRelationApply
 from cognite.client.exceptions import CogniteAPIError
 from cognite.client.utils.useful_types import SequenceNotStr
 from rich import print
@@ -94,6 +93,7 @@ from cognite_toolkit._cdf_tk.resource_classes import (
     ContainerYAML,
     DataModelYAML,
     EdgeYAML,
+    GraphQLDataModelYAML,
     NodeYAML,
     SpaceYAML,
     ViewYAML,
@@ -106,12 +106,11 @@ from cognite_toolkit._cdf_tk.utils import (
     load_yaml_inject_variables,
     quote_int_value_by_key_in_yaml,
     safe_read,
+    sanitize_filename,
     to_diff,
-    to_directory_compatible,
 )
 from cognite_toolkit._cdf_tk.utils.cdf import iterate_instances
 from cognite_toolkit._cdf_tk.utils.diff_list import diff_list_identifiable, dm_identifier
-from cognite_toolkit._cdf_tk.utils.tarjan import tarjan
 
 from .auth import GroupAllScopedCRUD
 
@@ -487,7 +486,7 @@ class ContainerCRUD(ResourceContainerCRUD[ContainerId, ContainerApply, Container
 
     @classmethod
     def as_str(cls, id: ContainerId) -> str:
-        return to_directory_compatible(f"{id.space}_{id.external_id}")
+        return sanitize_filename(f"{id.space}_{id.external_id}")
 
 
 class ViewCRUD(ResourceCRUD[ViewId, ViewApply, View, ViewApplyList, ViewList]):
@@ -651,78 +650,8 @@ class ViewCRUD(ResourceCRUD[ViewId, ViewApply, View, ViewApplyList, ViewList]):
             if self._is_auto_retryable(e1):
                 # Fallback to creating one by one if the error is auto-retryable.
                 return self._fallback_create_one_by_one(items, e1)
-            elif self._is_false_not_exists(e1, {item.as_id() for item in items}):
-                return self._try_to_recover_coupled(items, e1)
             raise
 
-    @staticmethod
-    def _is_false_not_exists(e: CogniteAPIError, request_views: set[ViewId]) -> bool:
-        if "not exist" not in e.message and 400 <= e.code < 500:
-            return False
-        results = re.findall(r"'([a-zA-Z0-9_-]+):([a-zA-Z0-9_]+)/([.a-zA-Z0-9_-]+)'", e.message)
-        if not results:
-            # No view references in the message
-            return False
-        error_message_views = {ViewId(*result) for result in results}
-        return error_message_views.issubset(request_views)
-
-    def _try_to_recover_coupled(self, items: Sequence[ViewApply], original_error: CogniteAPIError) -> ViewList:
-        """The /models/views endpoint can give faulty 400 about missing views that are part of the request.
-
-        This method tries to recover from such errors by identifying the strongly connected components in the graph
-        defined by the implements and through properties of the views. We then create the components in topological
-        order.
-
-        Args:
-            items: The items that failed to create.
-            original_error: The original error that was raised. If the problem is not recoverable, this error is raised.
-
-        Returns:
-            The views that were created.
-
-        """
-        views_by_id = {self.get_id(item): item for item in items}
-        parents_by_child: dict[ViewId, set[ViewId]] = {
-            view_id: {parent for parent in view.implements or [] if parent in views_by_id}
-            for view_id, view in views_by_id.items()
-        }
-        # Check for cycles in the implements graph
-        try:
-            TopologicalSorter(parents_by_child).static_order()
-        except CycleError as e:
-            raise ToolkitCycleError(f"Failed to deploy views. This likely due to a cycle in implements. {e.args[1]}")
-
-        dependencies_by_id: dict[ViewId, set[ViewId]] = defaultdict(set)
-        for view_id, view in views_by_id.items():
-            dependencies_by_id[view_id].update([parent for parent in view.implements or [] if parent in views_by_id])
-            for properties in (view.properties or {}).values():
-                if isinstance(properties, ReverseDirectRelationApply):
-                    if isinstance(properties.through.source, ViewId) and properties.through.source in views_by_id:
-                        dependencies_by_id[view_id].add(properties.through.source)
-
-        LowSeverityWarning(
-            f"Failed to create {len(items)} views: {escape(original_error.message)}.\nAttempting to recover..."
-        ).print_warning(include_timestamp=True, console=self.console)
-        created = ViewList([])
-        for strongly_connected in tarjan(dependencies_by_id):
-            to_create = [views_by_id[view_id] for view_id in strongly_connected]
-            try:
-                created_set = self.client.data_modeling.views.apply(to_create)
-            except CogniteAPIError as error:
-                self.client.data_modeling.views.delete(created.as_ids())
-                HighSeverityWarning(
-                    f"Recovering attempt failed. Could not create views {self.get_ids(to_create)}: "
-                    f"{escape(error.message)}.\n Raising original error."
-                ).print_warning(console=self.console)
-                raise original_error
-            created.extend(created_set)
-        message = f"Recovery attempt succeeded. Created {len(created)} views."
-        if self.console:
-            self.console.print(message)
-        else:
-            print(message)
-        return created
-
     @staticmethod
     def _is_auto_retryable(e: CogniteAPIError) -> bool:
         return isinstance(e.extra, dict) and "isAutoRetryable" in e.extra and e.extra["isAutoRetryable"]
@@ -870,7 +799,7 @@ class ViewCRUD(ResourceCRUD[ViewId, ViewApply, View, ViewApplyList, ViewList]):
 
     @classmethod
     def as_str(cls, id: ViewId) -> str:
-        return to_directory_compatible(id.external_id)
+        return sanitize_filename(id.external_id)
 
     def _lookup_views(self, view_ids: list[ViewId]) -> dict[ViewId, View]:
         """Looks up views by their IDs and caches them."""
@@ -1038,7 +967,7 @@ class DataModelCRUD(ResourceCRUD[DataModelId, DataModelApply, DataModel, DataMod
 
     @classmethod
     def as_str(cls, id: DataModelId) -> str:
-        return to_directory_compatible(id.external_id)
+        return sanitize_filename(id.external_id)
 
 
 @final
@@ -1211,7 +1140,7 @@ class NodeCRUD(ResourceContainerCRUD[NodeId, NodeApply, Node, NodeApplyList, Nod
 
     @classmethod
     def as_str(cls, id: NodeId) -> str:
-        return to_directory_compatible(f"{id.space}_{id.external_id}")
+        return sanitize_filename(f"{id.space}_{id.external_id}")
 
 
 class GraphQLCRUD(
@@ -1228,6 +1157,7 @@ class GraphQLCRUD(
     kind = "GraphQLSchema"
     dependencies = frozenset({SpaceCRUD, ContainerCRUD})
     item_name = "views"
+    yaml_cls = GraphQLDataModelYAML
     _doc_url = "Data-models/operation/createDataModels"
     _hash_name = "CDFToolkitHash:"
 
