@@ -81,6 +81,10 @@ class Flags(Enum):
         visible=True,
         description="Enables the support for the search config command",
     )
+    FUNCTION_COGNITE_FILE = FlagMetadata(
+        visible=True,
+        description="Enables support for using Cognite Files to store function code",
+    )
 
     def is_enabled(self) -> bool:
         return FeatureFlag.is_enabled(self)
