@@ -1,20 +1,19 @@
 from abc import ABC, abstractmethod
 from collections.abc import Iterable, Mapping, Sequence
 from dataclasses import dataclass
-from pathlib import Path
-from typing import ClassVar, Generic
+from typing import ClassVar, Generic, TypeVar
 
 from cognite.client.data_classes._base import (
     T_CogniteResourceList,
 )
-from rich.console import Console
 
 from cognite_toolkit._cdf_tk.client import ToolkitClient
 from cognite_toolkit._cdf_tk.exceptions import ToolkitNotImplementedError
-from cognite_toolkit._cdf_tk.utils.collection import chunker_sequence
 from cognite_toolkit._cdf_tk.utils.fileio import SchemaColumn
 from cognite_toolkit._cdf_tk.utils.http_client import HTTPClient, HTTPMessage, ItemsRequest
-from cognite_toolkit._cdf_tk.utils.useful_types import T_ID, JsonVal, T_Selector, T_WritableCogniteResourceList
+from cognite_toolkit._cdf_tk.utils.useful_types import T_ID, JsonVal, T_WritableCogniteResourceList
+
+from .selectors import DataSelector
 
 
 @dataclass
@@ -22,6 +21,10 @@ class StorageIOConfig:
     kind: str
     folder_name: str
     value: JsonVal
+    filename: str | None = None
+
+
+T_Selector = TypeVar("T_Selector", bound=DataSelector)
 
 
 class StorageIO(ABC, Generic[T_ID, T_Selector, T_CogniteResourceList, T_WritableCogniteResourceList]):
@@ -51,6 +54,7 @@ class StorageIO(ABC, Generic[T_ID, T_Selector, T_CogniteResourceList, T_Writable
     CHUNK_SIZE: int
     UPLOAD_ENDPOINT: ClassVar[str]
     UPLOAD_EXTRA_ARGS: ClassVar[Mapping[str, JsonVal] | None] = None
+    BASE_SELECTOR: ClassVar[type[DataSelector]]
 
     def __init__(self, client: ToolkitClient) -> None:
         self.client = client
@@ -90,22 +94,14 @@ class StorageIO(ABC, Generic[T_ID, T_Selector, T_CogniteResourceList, T_Writable
         """
         raise NotImplementedError()
 
-    @abstractmethod
-    def upload_items(self, data_chunk: T_CogniteResourceList, selector: T_Selector) -> None:
-        """Upload a chunk of data to the storage.
-
-        Args:
-            data_chunk: The chunk of data to upload, which should be a list of writable Cognite resources.
-            selector: The selection criteria to identify where to upload the data.
-        """
-        raise NotImplementedError()
-
-    def upload_items_force(
+    def upload_items(
         self, data_chunk: T_CogniteResourceList, http_client: HTTPClient, selector: T_Selector | None = None
     ) -> Sequence[HTTPMessage]:
         """Upload a chunk of data to the storage using a custom HTTP client.
         This ensures that even if one item in the chunk fails, the rest will still be uploaded.
 
+        This assumes that the data_chunk is respecting the CHUNK_SIZE of the storage.
+
         Args:
             data_chunk: The chunk of data to upload, which should be a list of writable Cognite resources.
             http_client: The custom HTTP client to use for the upload.
@@ -113,24 +109,21 @@ class StorageIO(ABC, Generic[T_ID, T_Selector, T_CogniteResourceList, T_Writable
         """
         if not hasattr(self, "UPLOAD_ENDPOINT"):
             raise ToolkitNotImplementedError(f"Upload not implemented for {self.KIND} storage.")
+        if len(data_chunk) > self.CHUNK_SIZE:
+            raise ValueError(f"Data chunk size {len(data_chunk)} exceeds the maximum CHUNK_SIZE of {self.CHUNK_SIZE}.")
 
         config = http_client.config
-        results: list[HTTPMessage] = []
-        for batch in chunker_sequence(data_chunk, self.CHUNK_SIZE):
-            batch_results = http_client.request_with_retries(
-                message=ItemsRequest(
-                    endpoint_url=config.create_api_url(self.UPLOAD_ENDPOINT),
-                    method="POST",
-                    # The dump method from the PySDK always returns JsonVal, but mypy cannot infer that
-                    items=batch.dump(camel_case=True),  # type: ignore[arg-type]
-                    extra_body_fields=dict(self.UPLOAD_EXTRA_ARGS or {}),
-                    as_id=self.as_id,
-                )
+        return http_client.request_with_retries(
+            message=ItemsRequest(
+                endpoint_url=config.create_api_url(self.UPLOAD_ENDPOINT),
+                method="POST",
+                # The dump method from the PySDK always returns JsonVal, but mypy cannot infer that
+                items=data_chunk.dump(camel_case=True),  # type: ignore[arg-type]
+                extra_body_fields=dict(self.UPLOAD_EXTRA_ARGS or {}),
+                as_id=self.as_id,
             )
-            results.extend(batch_results)
-        return results
+        )
 
-    @abstractmethod
     def data_to_json_chunk(self, data_chunk: T_WritableCogniteResourceList) -> list[dict[str, JsonVal]]:
         """Convert a chunk of data to a JSON-compatible format.
 
@@ -141,7 +134,7 @@ class StorageIO(ABC, Generic[T_ID, T_Selector, T_CogniteResourceList, T_Writable
             A list of dictionaries representing the data in a JSON-compatible format.
 
         """
-        raise NotImplementedError()
+        return data_chunk.as_write().dump(camel_case=True)
 
     @abstractmethod
     def json_chunk_to_data(self, data_chunk: list[dict[str, JsonVal]]) -> T_CogniteResourceList:
@@ -154,34 +147,17 @@ class StorageIO(ABC, Generic[T_ID, T_Selector, T_CogniteResourceList, T_Writable
         """
         raise NotImplementedError()
 
+
+class ConfigurableStorageIO(StorageIO[T_ID, T_Selector, T_CogniteResourceList, T_WritableCogniteResourceList], ABC):
     @abstractmethod
     def configurations(self, selector: T_Selector) -> Iterable[StorageIOConfig]:
         """Return configurations for the storage item."""
         raise NotImplementedError()
 
-    @abstractmethod
-    def load_selector(self, datafile: Path) -> T_Selector:
-        """Load the selector from adjacent filepath."""
-        raise NotImplementedError()
 
-    @abstractmethod
-    def ensure_configurations(self, selector: T_Selector, console: Console | None = None) -> None:
-        """Ensure that the necessary configurations for the storage item are in place.
-
-        This method should create the necessary configurations in CDF if they do not exist.
-        For example, for RAW tables, this will create the RAW database and table.
-
-        For asset-centric storage, this will create labels and data sets.
-
-        Args:
-            selector: The selection criteria to find the data.
-            console: An optional console for outputting messages during the configuration process.
-
-        """
-        raise NotImplementedError()
-
-
-class TableStorageIO(StorageIO[T_ID, T_Selector, T_CogniteResourceList, T_WritableCogniteResourceList], ABC):
+class TableStorageIO(
+    ConfigurableStorageIO[T_ID, T_Selector, T_CogniteResourceList, T_WritableCogniteResourceList], ABC
+):
     @abstractmethod
     def get_schema(self, selector: T_Selector) -> list[SchemaColumn]:
         """Get the schema of the table associated with the given selector.
