@@ -36,7 +36,7 @@ from cognite_toolkit._cdf_tk.tk_warnings import (
 )
 from cognite_toolkit._cdf_tk.utils import humanize_collection
 
-from .file import to_directory_compatible
+from .file import sanitize_filename
 from .sql_parser import SQLParser
 
 if sys.version_info < (3, 11):
@@ -103,10 +103,14 @@ def iterate_instances(
     # By making the sort be on external_id, Postgres should pick the index that's on (project_id, space, external_id)
     # WHERE deleted_at IS NULL. In other words, avoiding soft deleted instances.
     body["sort"] = [
+        {
+            "property": [instance_type, "space"],
+            "direction": "ascending",
+        },
         {
             "property": [instance_type, "externalId"],
             "direction": "ascending",
-        }
+        },
     ]
     url = f"/api/{client._API_VERSION}/projects/{client.config.project}/models/instances/list"
     if space:
@@ -406,7 +410,7 @@ class ThrottlerState:
     @classmethod
     def _filepath(cls, project: str) -> Path:
         # Enure the project name is a valid string for hashing.
-        project = to_directory_compatible(project)
+        project = sanitize_filename(project)
         filename = cls._FILENAME.format(project=project)
         return Path(tempfile.gettempdir()) / filename
 
