@@ -22,7 +22,12 @@ from cognite_toolkit._cdf_tk.tk_warnings import (
 )
 from cognite_toolkit._cdf_tk.tk_warnings.fileread import ResourceFormatWarning
 
-__all__ = ["validate_data_set_is_set", "validate_modules_variables", "validate_resource_yaml"]
+__all__ = [
+    "humanize_validation_error",
+    "validate_data_set_is_set",
+    "validate_modules_variables",
+    "validate_resource_yaml",
+]
 
 
 T_BaseModel = TypeVar("T_BaseModel", bound=BaseModel)
@@ -136,18 +141,17 @@ def validate_resource_yaml_pydantic(
 
     """
     warning_list: WarningList = WarningList()
-    if isinstance(data, dict):
-        try:
+    try:
+        if isinstance(data, dict):
             validation_cls.model_validate(data, strict=True)
-        except ValidationError as e:
-            warning_list.append(ResourceFormatWarning(source_file, tuple(_humanize_validation_error(e))))
-    elif isinstance(data, list):
-        try:
+        elif isinstance(data, list):
             TypeAdapter(list[validation_cls]).validate_python(data)  # type: ignore[valid-type]
-        except ValidationError as e:
-            warning_list.append(ResourceFormatWarning(source_file, tuple(_humanize_validation_error(e))))
-    else:
-        raise ValueError(f"Expected a dictionary or list of dictionaries, got {type(data)}.")
+        else:
+            raise ValueError(f"Expected a dictionary or list of dictionaries, got {type(data)}.")
+    except ValidationError as e:
+        printable_errors = tuple(humanize_validation_error(e))
+        if printable_errors:
+            warning_list.append(ResourceFormatWarning(source_file, printable_errors))
     return warning_list
 
 
@@ -168,10 +172,10 @@ def instantiate_class(
     try:
         return validation_cls.model_validate(data, strict=strict)
     except ValidationError as e:
-        return ResourceFormatWarning(source_file, tuple(_humanize_validation_error(e)))
+        return ResourceFormatWarning(source_file, tuple(humanize_validation_error(e)))
 
 
-def _humanize_validation_error(error: ValidationError) -> list[str]:
+def humanize_validation_error(error: ValidationError) -> list[str]:
     """Converts a ValidationError to a human-readable format.
 
     This overwrites the default error messages from Pydantic to be better suited for Toolkit users.
