@@ -416,7 +416,7 @@ authentication:
     )
 
 
-class TestResourceLoaders:
+class TestResourceCRUDs:
     # The HostedExtractorSourceLoader does not support parameter spec.
     @pytest.mark.parametrize(
         "loader_cls",
@@ -519,6 +519,11 @@ class TestResourceLoaders:
 
         assert isinstance(dumped, dict)
 
+    def test_yaml_cls_is_set(self) -> None:
+        missing = [loader_cls.__name__ for loader_cls in RESOURCE_CRUD_LIST if loader_cls.yaml_cls is None]
+
+        assert not missing, f"The following loaders are missing yaml_cls: {missing}"
+
 
 class TestLoaders:
     def test_unique_display_names(self, env_vars_with_client: EnvironmentVariables):
