@@ -14,8 +14,8 @@ from cognite_toolkit._cdf_tk._parameters import (
     read_parameter_from_init_type_hints,
     read_parameters_from_dict,
 )
-from cognite_toolkit._cdf_tk.loaders import RESOURCE_LOADER_LIST, ResourceLoader
-from cognite_toolkit._cdf_tk.loaders._resource_loaders.location_loaders import LocationFilterLoader
+from cognite_toolkit._cdf_tk.cruds import RESOURCE_CRUD_LIST, ResourceCRUD
+from cognite_toolkit._cdf_tk.cruds._resource_cruds.location import LocationFilterCRUD
 
 
 class TestReadParameterFromTypeHints:
@@ -151,9 +151,9 @@ class TestReadParameterFromTypeHints:
 
         assert sorted(actual_parameters) == sorted(expected_parameters)
 
-    @pytest.mark.parametrize("loader_cls", RESOURCE_LOADER_LIST)
-    def test_compatible_with_loaders(self, loader_cls: type[ResourceLoader]) -> None:
-        if loader_cls is LocationFilterLoader:
+    @pytest.mark.parametrize("loader_cls", RESOURCE_CRUD_LIST)
+    def test_compatible_with_loaders(self, loader_cls: type[ResourceCRUD]) -> None:
+        if loader_cls is LocationFilterCRUD:
             # TODO: https://cognitedata.atlassian.net/browse/CDF-22363
             pytest.skip(f"Skipping {loader_cls} because get_write_cls_parameter_spec fails for some reason")
 
