@@ -9,7 +9,7 @@ import random
 import re
 import string
 import typing
-from collections.abc import Callable
+from collections.abc import Callable, Hashable
 from datetime import date, datetime
 from pathlib import Path
 from types import UnionType
@@ -117,16 +117,16 @@ def mock_read_yaml_file(
 
     monkeypatch.setattr("cognite_toolkit._cdf_tk.utils.load_yaml_inject_variables", fake_load_yaml_inject_variables)
     monkeypatch.setattr(
-        "cognite_toolkit._cdf_tk.loaders._base_loaders.load_yaml_inject_variables", fake_load_yaml_inject_variables
+        "cognite_toolkit._cdf_tk.cruds._base_cruds.load_yaml_inject_variables", fake_load_yaml_inject_variables
     )
     for module in [
-        "classic_loaders",
-        "datamodel_loaders",
-        "industrial_tool_loaders",
-        "transformation_loaders",
+        "classic",
+        "datamodel",
+        "industrial_tool",
+        "transformation",
     ]:
         monkeypatch.setattr(
-            f"cognite_toolkit._cdf_tk.loaders._resource_loaders.{module}.load_yaml_inject_variables",
+            f"cognite_toolkit._cdf_tk.cruds._resource_cruds.{module}.load_yaml_inject_variables",
             fake_load_yaml_inject_variables,
         )
 
@@ -367,6 +367,10 @@ class FakeCogniteResourceGenerator:
         # Handle containers
         args = get_args(type_)
         first_not_none = next((arg for arg in args if arg is not None), None)
+        if get_origin(first_not_none) is Callable and type(None) in args:
+            # If the type is Optional[Callable], we cannot generate a value, so we return None
+            return None
+
         if container_type in UNION_TYPES:
             return self.create_value(first_not_none)
         elif container_type is typing.Literal:
@@ -449,6 +453,8 @@ class FakeCogniteResourceGenerator:
             return self.create_value(type_.__bound__)
         elif inspect.isclass(type_) and issubclass(type_, CogniteResourceList):
             return type_([self.create_value(type_._RESOURCE) for _ in range(self._random.randint(1, 3))])
+        elif type_ is Hashable:
+            return "my_hashable"
         elif inspect.isclass(type_):
             return self.create_instance(type_)
         elif type(type_) is dataclasses.InitVar:
@@ -534,8 +540,7 @@ class MockQuestionary:
         return [choice.value for choice in choices]
 
 
-def find_resources(resource: str, resource_dir: str | None = None):
-    base = COMPLETE_ORG / MODULES
+def find_resources(resource: str, resource_dir: str | None = None, base: Path = COMPLETE_ORG / MODULES):
     for path in base.rglob(f"*{resource}.yaml"):
         if resource_dir and resource_dir not in path.parts:
             continue
